% Main script running the simulation and processing output
%
% Author:   Peter Tenfjord and Martin Bakke 
% Date:     01.06.2017

clear; close all; clc;

addpath('Blocks')
addpath('Data_Source')
addpath('Functions')
addpath('Pre_Processing')
addpath('ReiseSim')


%% Preprocessing
% Read the vessel input file and make the parameters available for the
% Simulink model. All blocks that want to access these data: Data Editor-->Input-->Parameter

filename = 'Vessel_Info.xlsx';
xlRange = sprintf('B%d:BC%d',5,5); %The numbers at the end specify rows in the file, set them according to what info you want to use
[num,txt] = xlsread(filename,xlRange);

% Read engine data into EngDat input file
% Configure the engine_pre function according to your system (Functions
% folder)
EngDat=engine_pre();

RoundTrip=0; % set to 1 if simulating roundtrip, 0 otherwise
SetNumRTs=1; % set the number of roundtrips you want to simulate, set to 1 if RoundTrip=0;

seed=2; %choose the seed to control random number generation
Random_Values=generate_random_matrix(seed,SetNumRTs); %generate random number matrix

%% Still water Resistance table for shipx, waveres
%Uncomment the next line if you are using ShipX in the hydrodynamic module
%Resistance_Table = waveRes ();

%% Generate markov chains (MC) for weather parameters
N=6; % Number of legs. Configure this according to route (Get Route function)
states = 9; % Wanted number of states for markov chain
FirstMonth = 11; % First chosen month for weather data
LastMonth = 12; % Last month for weather data

% Preallocation of matrices; This is done in all sub-codes, as Simulink
% requires it in order to run. 
% MC_... matrices are markov matrices ( states x states x leg number)
% ...Range is the value range for the respective period and leg chosen
MC_Hs_sea = zeros(states,states,N); Hs_seaRange = zeros(states,N);
MC_Hsd_sea = zeros(states,states,N); Hsd_seaRange = zeros(states,N);
MC_Tp_sea = zeros(states,states,N); Tp_seaRange = zeros(states,N);
MC_Hs_swl = zeros(states,states,N); Hs_swlRange = zeros(states,N);
MC_Hsd_swl = zeros(states,states,N); Hsd_swlRange = zeros(states,N);
MC_Tp_swl = zeros(states,states,N); Tp_swlRange = zeros(states,N);
MC_u10 = zeros(states,states,N); u10_Range = zeros(states,N);
MC_u10d = zeros(states,states,N); u10d_Range = zeros(states,N);


for j=1:N
   
    str=sprintf('Leg%d.csv',j);
    
    % Wind-Sea values
    [MC_Hs_sea(:,:,j),Hs_seaRange(:,j)] = MC_param(str,states,FirstMonth,LastMonth,12);  
    [MC_Hsd_sea(:,:,j),Hsd_seaRange(:,j)] = MC_param(str,states,FirstMonth,LastMonth,13);  
    [MC_Tp_sea(:,:,j),Tp_seaRange(:,j)] = MC_param(str,states,FirstMonth,LastMonth,14); 
    
    % Swell values
    [MC_Hs_swl(:,:,j),Hs_swlRange(:,j)] = MC_param(str,states,FirstMonth,LastMonth,17);  
    [MC_Hsd_swl(:,:,j),Hsd_swlRange(:,j)] = MC_param(str,states,FirstMonth,LastMonth,18);  
    [MC_Tp_swl(:,:,j),Tp_swlRange(:,j)] = MC_param(str,states,FirstMonth,LastMonth,19);  
    
    % Wind values
    [MC_u10(:,:,j),u10_Range(:,j)] = MC_param(str,states,FirstMonth,LastMonth,5); 
    [MC_u10d(:,:,j),u10d_Range(:,j)] = MC_param(str,states,FirstMonth,LastMonth,6); 
    
end

    clear str j N 
%% Prepare [course, distnm]
%Run the get_route function to automatically attain waypoints (wpts)
%get_route(origin, destination,ship_speed)

%Or manually input waypoints below
% wpts= [];
    %The following waypoints can be used as a guide, they follow a route
    %from China to Panama
wpts= [32.8502 121.2880;      
       33.5522 128.4185;
       41.2058 138.4851;
       46.9253 -179.1512;
       41.8339 -150.2097;
       15.3330 -99.8767;
       7.1122 -79.7435];
   
% If waypoints are inputted manually, run the following
% to store the course and distance of the legs, to be used in simulation
[course_legs, distnm_legs] = legs(wpts,'rh');

%If using a roundtrip model, uncomment the following

%if RoundTrip==true
%wptsBA=flip(wpts);
%[course_legsBA,distnm_legsBA]=legs(wptsBA,'rh');
%end

%% Set the required, or intended, speed for the ship

v_req_start=14.5; %[kts]


%% Execute the simulation

if RoundTrip==true
    tic
sim('Main_RT.slx');
    toc
else
    tic
sim('Main.slx');
    toc
end
    
    
%% Process the output from simulation
%  This data is from the "To Workspace" blocks in the model
t = tout;                        % Time steps of events
Hs_sea = MetOcean.data(:,1);     % Significant wave height, Sea based [m]
relD_sea = MetOcean.data(:,2);   % Direction of wave, Sea based  [deg]
Tp_sea = MetOcean.data(:,3);     % Peak period of wave, Sea based [s]
Hs_swl = MetOcean.data(:,4);     % Significant wave height, Swell based [m]
relD_swl = MetOcean.data(:,5);   % Direction of wave, Swell based  [deg]
Tp_swl = MetOcean.data(:,6);     % Peak period of wave, Swell based [s]
u10 = MetOcean.data(:,7);        % Wind Speed [m/s]
relD_u10 = MetOcean.data(:,8);   % Relative direction of wind [deg]
Raw = Raw.data;                  % Added resistance [N]
Res = R.data;                    % Calm water resistance [N]
Raa = Raa.data;                  % Air resistance [N]
Rtot = Res+Raw+Raa;              % Total resistance [N]
Raw_Res = (Raw./Res)*100;        % Added resistance as a percentage of calm water resistance [%]
ME1FC = engroom.data(:,1);       % Main Engine fuel consumption [kg/h]
ME1PB = engroom.data(:,2);       % Main Engine Break Power Production [W]
ME1X = engroom.data(:,3);        % Main Engine Loading degree [%]
V_att = V_act.data;              % The actual attainable speed, from Kwon [kts]
LatPts = LatLon.data(:,1);       % The Simulated Latitudes
LonPts = LatLon.data(:,2);       % The Simulated Longitudes

%Gather output data in tabular form
if RoundTrip==true
VoyStep=voyage_step.data;    % Ship status (in port or sailing) during simulation
%Uncomment the following if there are different lengths in vectors to be gathered in table
%VoyStep=[VoyStep;2];
T=table(t,VoyStep,V_att,Hs_sea,relD_sea,Tp_sea,Hs_swl,relD_swl,Tp_swl,u10,relD_u10,Raw,Res,Raa,Rtot,Raw_Res,ME1FC,ME1PB,ME1X);
else
T=table(t,V_att,Hs_sea,relD_sea,Tp_sea,Hs_swl,relD_swl,Tp_swl,u10,relD_u10,Raw,Res,Raa,Rtot,Raw_Res,ME1FC,ME1PB,ME1X);
end


%% Calculate averages for different parameters

if RoundTrip==true
    R_total = zeros(1,SetNumRTs*2);
    PB_total = zeros(1,SetNumRTs*2);
    Fuel_Cons_Total = zeros(1,SetNumRTs*2);
    SailTimes = zeros(1,SetNumRTs*2);
    num_steps = zeros(1,SetNumRTs*2);
    j = 1;
for j = 1:SetNumRTs*2
    for k = 1:numel(t)
    if VoyStep(k) == j
            num_steps(j) = num_steps(j)+1;
            SailTimes(j) = SailTimes(j)+1;
            R_total(j) = R_total(j)+Rtot(k);
            PB_total(j) = PB_total(j)+ME1PB(k);
            Fuel_Cons_Total(j) = Fuel_Cons_Total(j) + ME1FC(k);
    end
    end
end

 Avg_Res=zeros(1,SetNumRTs*2);
 Avg_PB=zeros(1,SetNumRTs*2);
 
for i=1:SetNumRTs*2
 Avg_Res(i)=R_total(i)/num_steps(i);
 Avg_PB(i)=PB_total(i)/num_steps(i);
end
 
% Create summary table
SailTimes=SailTimes';
Avg_Res=Avg_Res';
Avg_PB=Avg_PB';
Fuel_Cons_Total=Fuel_Cons_Total';

Summary=table(SailTimes,Avg_Res,Avg_PB,Fuel_Cons_Total)
else
    
Tot_Sail_Time=numel(t);         % [h], as each time step represents 1 hour 
SailTimes=Tot_Sail_Time;
Avg_Res=mean(Rtot);             % Average total resistance [N]
Avg_PB=mean(ME1PB);             % average load on Main Engine [W]
Fuel_Cons_Total=sum(ME1FC)/1000;  % Total fuel consumption [t]
% Create summary table
Summary=table(SailTimes,Avg_Res,Avg_PB,Fuel_Cons_Total)
end





%% Plot Simulation results, Ship performance, and Voyage coordinates

Plot_RunMaster;

